require_relative '../spec_helper'

describe SimplifyArithmetic do

it "should turn [:arithmetic] ast with more than one operation into a series of nested arithmetic statements, following the excel precedence rules" do

input = <<END
A1\t[:arithmetic, [:number, 1], [:operator, :+], [:number, 2], [:operator, :+], [:number, 3]]
A2\t[:arithmetic, [:number, 1], [:operator, :+], [:number, 2], [:operator, :-], [:number, 3]]
A3\t[:arithmetic, [:number, 1], [:operator, :/], [:number, 2], [:operator, :*], [:number, 3]]
A4\t[:arithmetic, [:number, 1], [:operator, :*], [:number, 2], [:operator, :^], [:number, 3]]
A5\t[:arithmetic, [:number, 1], [:operator, :/], [:number, 2], [:operator, :^], [:number, 3]]
A6\t[:arithmetic, [:number, 1], [:operator, :+], [:number, 2], [:operator, :*], [:number, 3]]
A7\t[:arithmetic, [:number, 1], [:operator, :+], [:number, 2], [:operator, :/], [:number, 3]]
A8\t[:arithmetic, [:number, 1], [:operator, :-], [:number, 2], [:operator, :*], [:number, 3]]
A9\t[:arithmetic, [:number, 1], [:operator, :-], [:number, 2], [:operator, :/], [:number, 3]]
A10\t[:arithmetic, [:number, 1], [:operator, :-], [:number, 2], [:operator, :/], [:number, 3],  [:operator, :+], [:number, 4]]
A11\t[:arithmetic, [:brackets, [:arithmetic, [:number, 1], [:operator, :+], [:cell, "$A$1"], [:operator, :*], [:number, 2]]], [:operator, :^], [:cell, "$A$2"], [:operator, :^], [:number, 1]]
A12\t[:brackets, [:arithmetic, [:cell, "N27"], [:operator, :*], [:cell, "N$18"], [:operator, :+], [:cell, "N28"], [:operator, :*], [:cell, "N$19"]]]
A13\t[:function, :MAX, [:number, 0.0], [:function, :IF, [:function, :AND, [:comparison, [:cell, :U438], [:comparator, :"="], [:number, 0.0]], [:comparison, [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U414], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U432]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U415], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U433]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U416], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U434]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U417], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U435]]]]], [:comparator, :<], [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U414], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U404]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U415], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U405]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U416], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U406]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U417], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U407]]]]]], [:comparison, [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U432]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U433]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U434]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U435]]]]], [:comparator, :<], [:arithmetic, [:cell, :"U$411"], [:operator, :-], [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:brackets, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U404]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:brackets, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U405]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:brackets, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U406]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:brackets, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U407]]]]]]]], [:arithmetic, [:function, :IF, [:comparison, [:cell, :"U$411"], [:comparator, :<=], [:function, :SUM, [:arithmetic, [:function, :IF, [:comparison, [:brackets, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:array, [:row, [:cell, :U404]], [:row, [:cell, :U405]], [:row, [:cell, :U406]], [:row, [:cell, :U407]]]]]], [:number, 0.0], [:arithmetic, [:arithmetic, [:cell, :"U$411"], [:operator, :-], [:function, :SUM, [:arithmetic, [:function, :IF, [:comparison, [:brackets, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:brackets, [:array, [:row, [:cell, :U404]], [:row, [:cell, :U405]], [:row, [:cell, :U406]], [:row, [:cell, :U407]]]]]]], [:operator, :*], [:cell, :U420]]], [:operator, :-], [:cell, :U426]], [:number, 0.0]]]
END

expected_output = <<END
A1\t[:arithmetic, [:arithmetic, [:number, 1], [:operator, :+], [:number, 2]], [:operator, :+], [:number, 3]]
A2\t[:arithmetic, [:arithmetic, [:number, 1], [:operator, :+], [:number, 2]], [:operator, :-], [:number, 3]]
A3\t[:arithmetic, [:arithmetic, [:number, 1], [:operator, :/], [:number, 2]], [:operator, :*], [:number, 3]]
A4\t[:arithmetic, [:number, 1], [:operator, :*], [:arithmetic, [:number, 2], [:operator, :^], [:number, 3]]]
A5\t[:arithmetic, [:number, 1], [:operator, :/], [:arithmetic, [:number, 2], [:operator, :^], [:number, 3]]]
A6\t[:arithmetic, [:number, 1], [:operator, :+], [:arithmetic, [:number, 2], [:operator, :*], [:number, 3]]]
A7\t[:arithmetic, [:number, 1], [:operator, :+], [:arithmetic, [:number, 2], [:operator, :/], [:number, 3]]]
A8\t[:arithmetic, [:number, 1], [:operator, :-], [:arithmetic, [:number, 2], [:operator, :*], [:number, 3]]]
A9\t[:arithmetic, [:number, 1], [:operator, :-], [:arithmetic, [:number, 2], [:operator, :/], [:number, 3]]]
A10\t[:arithmetic, [:arithmetic, [:number, 1], [:operator, :-], [:arithmetic, [:number, 2], [:operator, :/], [:number, 3]]], [:operator, :+], [:number, 4]]
A11\t[:arithmetic, [:arithmetic, [:arithmetic, [:number, 1], [:operator, :+], [:arithmetic, [:cell, "$A$1"], [:operator, :*], [:number, 2]]], [:operator, :^], [:cell, "$A$2"]], [:operator, :^], [:number, 1]]
A12\t[:arithmetic, [:arithmetic, [:cell, "N27"], [:operator, :*], [:cell, "N$18"]], [:operator, :+], [:arithmetic, [:cell, "N28"], [:operator, :*], [:cell, "N$19"]]]
A13\t[:function, :MAX, [:number, 0.0], [:function, :IF, [:function, :AND, [:comparison, [:cell, :U438], [:comparator, :"="], [:number, 0.0]], [:comparison, [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U414], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U432]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U415], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U433]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U416], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U434]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U417], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U435]]]]], [:comparator, :<], [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U414], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U404]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U415], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U405]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U416], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U406]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:cell, :U417], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U407]]]]]], [:comparison, [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U432]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U433]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U434]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :"="], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U435]]]]], [:comparator, :<], [:arithmetic, [:cell, :"U$411"], [:operator, :-], [:function, :SUM, [:array, [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U404]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U405]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U406]]], [:row, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:cell, :U407]]]]]]]], [:arithmetic, [:function, :IF, [:comparison, [:cell, :"U$411"], [:comparator, :<=], [:function, :SUM, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:array, [:row, [:cell, :U404]], [:row, [:cell, :U405]], [:row, [:cell, :U406]], [:row, [:cell, :U407]]]]]], [:number, 0.0], [:arithmetic, [:arithmetic, [:cell, :"U$411"], [:operator, :-], [:function, :SUM, [:arithmetic, [:function, :IF, [:comparison, [:array, [:row, [:cell, :U414]], [:row, [:cell, :U415]], [:row, [:cell, :U416]], [:row, [:cell, :U417]]], [:comparator, :<], [:cell, :U414]], [:number, 1.0], [:number, 0.0]], [:operator, :*], [:array, [:row, [:cell, :U404]], [:row, [:cell, :U405]], [:row, [:cell, :U406]], [:row, [:cell, :U407]]]]]], [:operator, :*], [:cell, :U420]]], [:operator, :-], [:cell, :U426]], [:number, 0.0]]]
END

input = StringIO.new(input)
output = StringIO.new
r = SimplifyArithmetic.new
r.replace(input,output)
output.string.should == expected_output

end # / it


end # / describe
